@@ -32,7 +32,7 @@
 #include <rcsc/soccer_math.h>
 #include <rcsc/geom/ray_2d.h>
 #include <rcsc/geom/rect_2d.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
