@@ -40,8 +40,10 @@
 
 #include <rcsc/formation/formation.h>
 #include <rcsc/player/logger.h>
-#include <rcsc/param/server_param.h>
-#include <rcsc/param/player_param.h>
+#include <rcsc/player/intercept_table.h>
+#include <rcsc/common/basic_client.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_param.h>
 #include <rcsc/player/audio_sensor.h>
 #include <rcsc/param/param_map.h>
 #include <rcsc/param/cmd_line_parser.h>
@@ -74,7 +76,7 @@ SamplePlayer::~SamplePlayer()
 
 */
 bool
-SamplePlayer::doInit( rcsc::CmdLineParser & cmd_parser )
+SamplePlayer::initImpl( rcsc::CmdLineParser & cmd_parser )
 {
     rcsc::ParamMap my_params;
 #if 0
@@ -84,10 +86,10 @@ SamplePlayer::doInit( rcsc::CmdLineParser & cmd_parser )
         ;
 #endif
 
-    if ( ! PlayerAgent::doInit( cmd_parser ) )
+    if ( ! rcsc::PlayerAgent::initImpl( cmd_parser ) )
     {
         my_params.printHelp( std::cout );
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return false;
     }
 
@@ -101,7 +103,7 @@ SamplePlayer::doInit( rcsc::CmdLineParser & cmd_parser )
     if ( ! M_strategy.read( config().configDir() ) )
     {
         std::cerr << "***ERROR*** Failed to read team strategy." << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return false;
     }
 
@@ -113,28 +115,13 @@ SamplePlayer::doInit( rcsc::CmdLineParser & cmd_parser )
     return true;
 }
 
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-SamplePlayer::initTeam()
-{
-    int number = config().playerNumber();
-    if ( number == 0 )
-    {
-        number = world().self().unum();
-        getConfig().setPlayerNumber( number );
-    }
-}
-
 /*-------------------------------------------------------------------*/
 /*!
   main decision
   virtual method in super class
 */
 void
-SamplePlayer::doAction()
+SamplePlayer::actionImpl()
 {
     if ( audioSensor().trainerMessage().time_ == world().time() )
     {
@@ -184,7 +171,7 @@ SamplePlayer::doAction()
                   << world().self().unum()
                   << " Error. Role is not registerd.\nExit ..."
                   << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
     if  ( ! role_ptr->hasFormation() )
@@ -193,7 +180,7 @@ SamplePlayer::doAction()
                   << world().self().unum()
                   << " Error. does not have formation.\nExit ..."
                   << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -265,7 +252,7 @@ SamplePlayer::doAction()
   virtual method in super clas
 */
 void
-SamplePlayer::doCommunication()
+SamplePlayer::communicationImpl()
 {
     if ( ! config().useCommunication()
          || world().gameMode().type() == rcsc::GameMode::BeforeKickOff
@@ -277,49 +264,49 @@ SamplePlayer::doCommunication()
 
     const rcsc::PlayerPtrCont & ball_near_mates = world().teammatesFromBall();
 
-
-    std::string say_msg;
+    std::string say_msg = effector().getSayMessage();
     say_msg.reserve( rcsc::ServerParam::i().playerSayMsgSize() );
 
-    bool reserved = false;
+    bool say_ball = false;
 
-    if ( ! effector().getSayMessage().empty() )
+    if ( ! say_msg.empty() )
     {
         rcsc::dlog.addText( rcsc::Logger::TEAM,
                             "say reserved message" );
-        debugClient().addMessage( "Say[%s]",
-                                  effector().getSayMessage().c_str() );
-        reserved = true;
     }
 
     // ball Info: seen at current
-    if ( ! reserved
+    if ( (int)say_msg.length() + 7 <= rcsc::ServerParam::i().playerSayMsgSize()
          && world().ball().posCount() == 0
          && world().ball().velCount() == 0 )
     {
-        rcsc::dlog.addText( rcsc::Logger::TEAM,
-                      "say chance" );
-        if ( world().self().isKickable()  // I am kickable
-             || ( ball_near_mates.empty()
-                  || ( ball_near_mates[0]->distFromBall() >
-                       world().ball().distFromSelf() - 3.0) )// I am closest to ball
-             || ( ball_near_mates.size() > 1
-                  && ( ball_near_mates[0]->distFromBall()  // closest to ball teammate is
-                       > rcsc::ServerParam::i().visibleDistance() - 0.5 ) // over vis dist
-                  && ( ball_near_mates[1]->distFromBall()
-                       > world().ball().distFromSelf() - 5.0 ) ) ) // I am second
+        if ( effector().queuedNextMyPos().dist( effector().queuedNextBallPos() )
+             < world().self().kickableArea() - 0.1 )
+        {
+            // if I am kickable in next cycle, ball info should not be said.
+        }
+        else if ( world().self().isKickable()  // I am kickable
+                  || ( ball_near_mates.empty()
+                       || ( ball_near_mates[0]->distFromBall() >
+                            world().ball().distFromSelf() - 3.0 ) ) // I am nearest to ball
+                  || ( ball_near_mates.size() > 1
+                       && ( ball_near_mates[0]->distFromBall()  // nearest to ball teammate is
+                            > rcsc::ServerParam::i().visibleDistance() - 0.5 ) // over vis dist
+                       && ( ball_near_mates[1]->distFromBall()
+                            > world().ball().distFromSelf() - 5.0 ) ) ) // I am second
         {
             // say
+            std::string msg; msg.reserve( 7 );
             audioSensor().encodeBall( effector().queuedNextBallPos(),
                                       effector().queuedNextBallVel(),
-                                      say_msg );
-            if ( ! say_msg.empty() )
+                                      msg );
+            if ( ! msg.empty() )
             {
                 rcsc::dlog.addText( rcsc::Logger::TEAM,
-                              "say ball status" );
-                debugClient().addMessage( "SayBall[%s]", say_msg.c_str() );
-                doSay( say_msg );
-                reserved = true;
+                                    "say ball status" );
+                debugClient().addMessage( "SayB" );
+                say_msg += msg;
+                say_ball = true;
             }
             else
             {
@@ -332,7 +319,7 @@ SamplePlayer::doCommunication()
     }
 
     // goalie info: ball is in chance area
-    if ( ! reserved
+    if ( (int)say_msg.length() + 6 <= rcsc::ServerParam::i().playerSayMsgSize()
          && world().ball().pos().x > 34.0
          && world().ball().pos().absY() < 25.0 )
     {
@@ -343,20 +330,19 @@ SamplePlayer::doCommunication()
              && opp_goalie->unum() > 0
              && opp_goalie->distFromSelf() < 25.0 )
         {
+            std::string msg; msg.reserve( 6 );
             audioSensor().encodeGoalie( opp_goalie->unum(),
                                         opp_goalie->pos(),
-                                        say_msg );
-            if ( ! say_msg.empty() )
+                                        msg );
+            if ( ! msg.empty() )
             {
                 rcsc::dlog.addText( rcsc::Logger::TEAM,
                                     "say goalie info to %d (%.1f %.1f)",
                                     opp_goalie->unum(),
                                     opp_goalie->pos().x,
                                     opp_goalie->pos().y );
-                debugClient().addMessage( "SayGoalie[%s]",
-                                          say_msg.c_str() );
-                doSay( say_msg );
-                reserved = true;
+                debugClient().addMessage( "SayG" );
+                say_msg += msg;
             }
             else
             {
@@ -368,6 +354,68 @@ SamplePlayer::doCommunication()
         }
     }
 
+    // intercept info
+    if ( world().ball().posCount() == 0
+         && world().ball().velCount() == 0
+         && (int)say_msg.length() + 3 <= rcsc::ServerParam::i().playerSayMsgSize() )
+    {
+        int self_min = world().interceptTable()->selfReachCycle();
+        int mate_min = world().interceptTable()->teammateReachCycle();
+        int opp_min = world().interceptTable()->opponentReachCycle();
+
+        if ( world().self().isKickable()
+             && ( effector().queuedNextMyPos().dist( effector().queuedNextBallPos() )
+                  > world().self().kickableArea() - 0.1 )
+             )
+        {
+            self_min = 100;
+        }
+
+        if ( self_min <= mate_min
+             && self_min <= opp_min
+             && self_min <= 8 )
+        {
+            std::string msg; msg.reserve( 3 );
+            audioSensor().encodeInterceptInfo( world().self().unum(),
+                                               self_min,
+                                               msg );
+            if ( ! msg.empty() )
+            {
+              rcsc::dlog.addText( rcsc::Logger::TEAM,
+                                  "say self intercept info %d",
+                                  self_min );
+              debugClient().addMessage( "SayI" );
+              say_msg += msg;
+            }
+        }
+
+    }
+
+    // offside line info
+    if ( (int)say_msg.length() + 3 <= rcsc::ServerParam::i().playerSayMsgSize()
+         && world().offsideLineCount() <= 3
+         && std::fabs( world().self().pos().x - world().offsideLineX() ) < 40.0 )
+    {
+        std::string msg; msg.reserve( 3 );
+        audioSensor().encodeOffsideLineX( world().offsideLineX(), msg );
+        if ( ! msg.empty() )
+        {
+            rcsc::dlog.addText( rcsc::Logger::TEAM,
+                                "say offside line %.1f",
+                                world().offsideLineX() );
+            debugClient().addMessage( "SayO" );
+            say_msg += msg;
+        }
+    }
+
+    if ( ! say_msg.empty() )
+    {
+        rcsc::dlog.addText( rcsc::Logger::TEAM,
+                            "say message [%s]",
+                            say_msg.c_str() );
+        debugClient().addMessage( "Say[%s]", say_msg.c_str() );
+        doSay( say_msg );
+    }
 
     // attention
 
@@ -379,9 +427,9 @@ SamplePlayer::doCommunication()
         }
     }
     else if ( ball_near_mates.front()->distFromBall() >
-              world().ball().distFromSelf() ) // I am closest
+              world().ball().distFromSelf() ) // I am nearest
     {
-        // if I am closest to ball, set attention to self nearest teammate
+        // if I am nearest to ball, set attention to self nearest teammate
         if ( ! world().teammatesFromSelf().empty() )
         {
             doAttentionto( world().ourSide(),
